﻿Public Class CalcYeardata
    Public WithEvents _filePathsConfig As New FilePathsConfig

    ' Event, das bei einem XML-Datei Fehler ausgelöst wird
    Public Event XMLError As EventHandler(Of String)
    ' Event, das bei erfolgreichem Lesen der XML-Daten ausgelöst wird
    Public Event XMLRead As EventHandler(Of DataTable)
    ' Event, das bei erfolgreichem Speichern der XML-Daten ausgelöst wird
    Public Event XMLSave As EventHandler
    ' Event, das bei erfolgreichem anlegen der XML-Daten ausgelöst wird
    Public Event XMLCreate As EventHandler

    ' DataSet und DataTables für die Verwaltung der XML-Daten.
    ' DataSet zur Speicherung der XML-Daten.
    Public dsWeather As New DataSet
    ' DataTable zur Speicherung der Jahresdaten.
    Public dtWeatherYear As New DataTable


    ''' <summary>
    ''' Lädt die XML-Daten aus der angegebenen Datei.
    ''' </summary>
    ''' <param name="yearfile">Der Dateipfad der XML-Datei.</param>
    Public Sub LoadXMLData(yearfile As String)
        MessageBox.Show(yearfile)
        If File.Exists(yearfile) Then
            Try
                yearfile = yearfile & "Abschluss" & _filePathsConfig.präfixPath
                Debug.WriteLine("XML-Dateiname/Pfad: " & yearfile)
                dsWeather.Clear()
                dsWeather.ReadXml(yearfile)
                If dsWeather.Tables.Count > 0 Then
                    dtWeatherYear = dsWeather.Tables(0)
                    ' Zum Testen: Den gekürzten Inhalt des ersten <WeatherData>-Tags ausgeben
                    Dim tableName As String = dsWeather.Tables(0).TableName
                    Debug.WriteLine("DataTable: " & tableName)
                    Dim xmlContent As String = dsWeather.GetXml()
                    Dim startIndex As Integer = xmlContent.IndexOf(tableName)
                    Dim endIndex As Integer = xmlContent.IndexOf("</WeatherData>", startIndex) + "</WeatherData>".Length
                    If startIndex >= 0 AndAlso endIndex > startIndex Then
                        Dim limitedContent As String = xmlContent.Substring(startIndex, endIndex - startIndex)
                        Debug.WriteLine("Inhalt gekürzt Lesen: " & limitedContent)
                    Else
                        Debug.WriteLine("Der XML-Inhalt enthält keinen gültigen Eintrag.")
                    End If
                    ' Test Ende
                Else
                    Debug.WriteLine("Die XML-Datei enthält keine Daten.")
                End If
                RaiseEvent XMLRead(Me, dtWeatherYear)
            Catch ex As Exception
                RaiseEvent XMLError(Me, ex.Message)
                Debug.WriteLine("Fehler beim Laden der XML-Daten: " & ex.Message)
                Debug.WriteLine("Details: " & ex.StackTrace)
            End Try
        End If
    End Sub

Public Sub CreateNewYear()
        ' Dateiname für das Jahres-XML-File erstellen
        Dim currentYear As Integer = DateTime.Now.Year
        Dim yearXmlFileName As String = "Abschluss-" & currentYear.ToString() & ".xml"

        If Not File.Exists(yearXmlFileName) Then
            Try
                Dim newYearData As New DataSet()
                Dim weatherDataTable As New DataTable("WeatherData")
                ' Spalten für die Wetterdaten hinzufügen
                weatherDataTable.Columns.Add("Datum", GetType(String))
                weatherDataTable.Columns.Add("Regen24h", GetType(Double))
                weatherDataTable.Columns.Add("SchneeTag", GetType(Double))
                weatherDataTable.Columns.Add("SchneeNacht", GetType(Double))
                weatherDataTable.Columns.Add("LuftfminTag", GetType(Double))
                weatherDataTable.Columns.Add("LuftfmaxTag", GetType(Double))
                weatherDataTable.Columns.Add("LuftfminNacht", GetType(Double))
                weatherDataTable.Columns.Add("LuftfmaxNacht", GetType(Double))
                weatherDataTable.Columns.Add("TempTag", GetType(Double))
                weatherDataTable.Columns.Add("TempNacht", GetType(Double))
                weatherDataTable.Columns.Add("WindTag", GetType(Double))
                weatherDataTable.Columns.Add("WindNacht", GetType(Double))
                ' Einträge für jeden Monat hinzufügen
                For month As Integer = 1 To 12 ' Durchlaufe alle Monate des Jahres
                    Dim newRow As DataRow = weatherDataTable.NewRow()
                    newRow("Datum") = month.ToString("00") & "-" & currentYear.ToString()
                    newRow("Regen24h") = 0
                    newRow("SchneeTag") = 0
                    newRow("SchneeNacht") = 0
                    newRow("LuftfminTag") = 0
                    newRow("LuftfmaxTag") = 0
                    newRow("LuftfminNacht") = 0
                    newRow("LuftfmaxNacht") = 0
                    newRow("TempTag") = 0
                    newRow("TempNacht") = 0
                    newRow("WindTag") = 0
                    newRow("WindNacht") = 0
                    weatherDataTable.Rows.Add(newRow)
                Next
                ' DataTable dem DataSet hinzufügen
                newYearData.Tables.Add(weatherDataTable)
                ' DataSet in XML-Datei schreiben
                newYearData.WriteXml(_filePathsConfig.yfilePath & yearXmlFileName)
                Debug.WriteLine("Neue XML-Datei für Jahr " & currentYear & " wurde erstellt.")
                ' Event auslösen
                'RaiseEvent XMLCreate(Me, EventArgs.Empty)
            Catch ex As Exception
                'RaiseEvent XMLError(Me, ex.Message)
                'Debug.WriteLine("Fehler beim Erstellen der XML-Datei für Jahr " & currentYear & ": " & ex.Message)
            End Try
        End If
    End Sub

    Public Sub ProcessWeatherData()
        ' Dateiname für das Jahres-XML-File erstellen
        Dim currentYear As Integer = DateTime.Now.Year
        Dim yearXmlFileName As String = "Abschluss-" & currentYear.ToString() & ".xml"
        ' DataTable für die Jahresdaten erstellen oder vorhandene DataTable laden
        'Dim dtWeatherYear As DataTable
        If File.Exists(_filePathsConfig.yfilePath) Then
            dtWeatherYear = New DataTable()
            Dim dataSet As New DataSet()
            Debug.WriteLine("JahrLese " & _filePathsConfig.yfilePath & yearXmlFileName)
            dataSet.ReadXml(_filePathsConfig.yfilePath & yearXmlFileName)
            dtWeatherYear = dataSet.Tables(0)
        Else
            dtWeatherYear = CreateInitialYearDataTable() ' Methode zum Erstellen einer leeren Jahresdaten-Tabelle aufrufen
        End If

        ' Durchschnitt für jeden Monat berechnen und in die Jahresdaten schreiben
        For Each monthXmlFile As String In GetMonthXmlFiles() ' Methode zum Abrufen der Monatsdateien aufrufen
            Dim dtWeatherMonth As New DataTable()

            ' Das Schema der DataTable festlegen
            dtWeatherMonth.Columns.Add("RegenTag", GetType(String))
            dtWeatherMonth.Columns.Add("SchneeTag", GetType(Double))
            dtWeatherMonth.Columns.Add("SchneeNacht", GetType(Double))
            dtWeatherMonth.Columns.Add("LuftfminTag", GetType(Double))
            dtWeatherMonth.Columns.Add("LuftfmaxTag", GetType(Double))
            dtWeatherMonth.Columns.Add("LuftfminNacht", GetType(Double))
            dtWeatherMonth.Columns.Add("LuftfmaxNacht", GetType(Double))
            dtWeatherMonth.Columns.Add("TempTag", GetType(Double))
            dtWeatherMonth.Columns.Add("TempNacht", GetType(Double))
            dtWeatherMonth.Columns.Add("WindTag", GetType(Double))
            dtWeatherMonth.Columns.Add("WindNacht", GetType(Double))

            dtWeatherMonth.ReadXml(monthXmlFile)

            ' Durchschnittliche Werte für den Monat berechnen
            Dim averageRegenTag As Double = dtWeatherMonth.AsEnumerable().Average(Function(row) row.Field(Of Integer)("RegenTag"))
            Dim averageSchneeTag As Double = dtWeatherMonth.AsEnumerable().Average(Function(row) row.Field(Of Integer)("SchneeTag"))
            Dim averageSchneeNacht As Double = dtWeatherMonth.AsEnumerable().Average(Function(row) row.Field(Of Integer)("SchneeNacht"))
            Dim averageLuftfminTag As Double = dtWeatherMonth.AsEnumerable().Average(Function(row) row.Field(Of Double)("LuftfminTag"))
            Dim averageLuftfmaxTag As Double = dtWeatherMonth.AsEnumerable().Average(Function(row) row.Field(Of Double)("LuftfmaxTag"))
            Dim averageLuftfminNacht As Double = dtWeatherMonth.AsEnumerable().Average(Function(row) row.Field(Of Double)("LuftfminNacht"))
            Dim averageLuftfmaxNacht As Double = dtWeatherMonth.AsEnumerable().Average(Function(row) row.Field(Of Double)("LuftfmaxNacht"))
            Dim averageTempTag As Double = dtWeatherMonth.AsEnumerable().Average(Function(row) row.Field(Of Integer)("TempTag"))
            Dim averageTempNacht As Double = dtWeatherMonth.AsEnumerable().Average(Function(row) row.Field(Of Integer)("TempNacht"))
            Dim averageWindTag As Double = dtWeatherMonth.AsEnumerable().Average(Function(row) row.Field(Of Integer)("WindTag"))
            Dim averageWindNacht As Double = dtWeatherMonth.AsEnumerable().Average(Function(row) row.Field(Of Integer)("WindNacht"))

            ' Datum für den Monat extrahieren
            Dim month As Integer = DateTime.ParseExact(Path.GetFileNameWithoutExtension(monthXmlFile), "MM-yyyy", CultureInfo.InvariantCulture).Month

            ' Überprüfen, ob bereits ein Eintrag für den Monat in der Jahresdaten-Tabelle vorhanden ist
            Dim existingRow As DataRow = dtWeatherYear.AsEnumerable().FirstOrDefault(Function(row) row.Field(Of Integer)("Datum") = month)

            If existingRow IsNot Nothing Then
                ' Eintrag für den Monat vorhanden - Durchschnittswerte aktualisieren
                existingRow("Regen24h") = averageRegenTag
                existingRow("SchneeTag") = averageSchneeTag
                existingRow("SchneeNacht") = averageSchneeNacht
                existingRow("LuftfminTag") = averageLuftfminTag
                existingRow("LuftfmaxTag") = averageLuftfmaxTag
                existingRow("LuftfminNacht") = averageLuftfminNacht
                existingRow("LuftfmaxNacht") = averageLuftfmaxNacht
                existingRow("TempTag") = averageTempTag
                existingRow("TempNacht") = averageTempNacht
                existingRow("WindTag") = averageWindTag
                existingRow("WindNacht") = averageWindNacht
            Else
                ' Eintrag für den Monat nicht vorhanden - neuen Eintrag erstellen
                Dim newRow As DataRow = dtWeatherYear.NewRow()
                newRow("Datum") = month
                newRow("Regen24h") = averageRegenTag
                newRow("SchneeTag") = averageSchneeTag
                newRow("SchneeNacht") = averageSchneeNacht
                newRow("LuftfminTag") = averageLuftfminTag
                newRow("LuftfmaxTag") = averageLuftfmaxTag
                newRow("LuftfminNacht") = averageLuftfminNacht
                newRow("LuftfmaxNacht") = averageLuftfmaxNacht
                newRow("TempTag") = averageTempTag
                newRow("TempNacht") = averageTempNacht
                newRow("WindTag") = averageWindTag
                newRow("WindNacht") = averageWindNacht
                dtWeatherYear.Rows.Add(newRow)
            End If
        Next

        ' DataTable als Teil eines DataSet erstellen
        Dim dataSetToSave As New DataSet()
        dataSetToSave.Tables.Add(dtWeatherYear)
        ' Daten als XML speichern (ohne Schema)
        MessageBox.Show(_filePathsConfig.yfilePath & yearXmlFileName)
        dataSetToSave.WriteXml(Path.Combine(_filePathsConfig.yfilePath & yearXmlFileName), XmlWriteMode.IgnoreSchema)

    End Sub

    Private Function GetMonthXmlFiles() As IEnumerable(Of String)
        Return Directory.GetFiles(_filePathsConfig.filePath, "*.xml")
    End Function

    Private Function CreateInitialYearDataTable() As DataTable
        Dim dtYear As New DataTable()
        ' Spalten für die Jahresdaten hinzufügen
        dtYear.Columns.Add("Datum", GetType(String))
        dtYear.Columns.Add("Regen24h", GetType(Double))
        dtYear.Columns.Add("WetterTag", GetType(String))
        dtYear.Columns.Add("WetterNacht", GetType(String))
        dtYear.Columns.Add("RegenTag", GetType(Double))
        dtYear.Columns.Add("SchneeTag", GetType(Double))
        dtYear.Columns.Add("SchneeNacht", GetType(Double))
        dtYear.Columns.Add("LuftfminTag", GetType(Double))
        dtYear.Columns.Add("LuftfmaxTag", GetType(Double))
        dtYear.Columns.Add("LuftfminNacht", GetType(Double))
        dtYear.Columns.Add("LuftfmaxNacht", GetType(Double))
        dtYear.Columns.Add("TempTag", GetType(Double))
        dtYear.Columns.Add("TempNacht", GetType(Double))
        dtYear.Columns.Add("WindTag", GetType(Double))
        dtYear.Columns.Add("WindNacht", GetType(Double))
        Return dtYear
    End Function
End Class