﻿Public Class DateManager
    Private _filePathsConfig As FilePathsConfig
    Private _xmlManager As XmlManager
    Private _dtpDatum As DateTimePicker

    ''' <summary>
    ''' Initialisiert eine neue Instanz der DateManager-Klasse.
    ''' </summary>
    ''' <param name="varsConst">Die Instanz der FilePathsConfig-Klasse.</param>
    ''' <param name="xmlManager">Die Instanz der XmlManager-Klasse.</param>
    ''' <param name="dtpDatum">Der DateTimePicker für das Datum.</param>
    Public Sub New(varsConst As FilePathsConfig, xmlManager As XmlManager, dtpDatum As DateTimePicker)
        _filePathsConfig = varsConst
        _xmlManager = xmlManager
        _dtpDatum = dtpDatum
    End Sub

    ''' <summary>
    ''' Lädt die XML-Daten entsprechend der aktuellen Konfiguration.
    ''' </summary>
    Public Sub Loadxml()
        If File.Exists(_filePathsConfig.XMLFileName) Then
            _xmlManager.LoadXMLData(_filePathsConfig.XMLFileName)
        Else
            Dim currentDate As DateTime = _dtpDatum.Value
            Dim daysInMonth As Integer = DateTime.DaysInMonth(currentDate.Year, currentDate.Month)
            ' Prüfen, ob es Monate gibt, für die noch keine Datei existiert
            Dim nextMonthExists As Boolean = File.Exists(GetNextMonthFileName())
            Dim prevMonthExists As Boolean = File.Exists(GetPreviousMonthFileName())
            ' Wenn die Datei für den aktuellen Monat nicht vorhanden ist, eine neue erstellen
            If Not File.Exists(_filePathsConfig.XMLFileName) Then
                ' Prüfen, ob die Datei für den aktuellen Monat erstellt werden soll
                If Not prevMonthExists Then
                    _xmlManager.CreateNewMonth(_filePathsConfig.XMLFileName, daysInMonth)
                End If
                _xmlManager.LoadXMLData(_filePathsConfig.XMLFileName)
            End If
        End If
    End Sub

    ''' <summary>
    ''' Gibt den Dateinamen für den nächsten Monat zurück.
    ''' </summary>
    ''' <returns>Der Dateiname für den nächsten Monat.</returns>
    Private Function GetNextMonthFileName() As String
        Dim currentDate As DateTime = _dtpDatum.Value
        Dim nextDate As DateTime = currentDate.AddMonths(1)
        Return String.Format("{0:00}-{1}.xml", nextDate.Month, nextDate.Year)
    End Function

    ''' <summary>
    ''' Gibt den Dateinamen für den vorherigen Monat zurück.
    ''' </summary>
    ''' <returns>Der Dateiname für den vorherigen Monat.</returns>
    Private Function GetPreviousMonthFileName() As String
        Dim currentDate As DateTime = _dtpDatum.Value
        Dim previousDate As DateTime = currentDate.AddMonths(-1)
        Return String.Format("{0:00}-{1}.xml", previousDate.Month, previousDate.Year)
    End Function
End Class
