﻿Imports System.Drawing.Printing
Imports System.Data

Public Class PrintingHelper
    Private printDocument As New PrintDocument()
    Private dataTable As DataTable
    Private title As String
    Private untertitel As String

    Private headerTopMargin As Integer = 1
    Private tableTopMargin As Integer = 2
    Private columnWidths As List(Of Integer) ' Liste der Spaltenbreiten
    Private columnHeight As Integer ' Höhe der Tabelle

    Private rowPosition As Integer
    Private pageCount As Integer
    Private totalPages As Integer

    Private RowsPerPage As Integer = 50 ' Anzahl der Zeilen pro Seite

    Public Sub New(data As DataTable, reportTitle As String, ut As String)
        dataTable = data
        title = reportTitle
        untertitel = ut
        totalPages = CInt(Math.Ceiling(dataTable.Rows.Count / CDbl(RowsPerPage)))
        ' Ereignishandler für das Druckereignis hinzufügen
        AddHandler printDocument.PrintPage, AddressOf PrintPageHandler
    End Sub

    Public Sub Print()
        ' Druckdialog anzeigen und Einstellungen auswählen
        Dim printDialog As New PrintDialog()
        printDialog.Document = printDocument
        If printDialog.ShowDialog() = DialogResult.OK Then
            ' Ausrichtung auf Querformat setzen
            printDocument.DefaultPageSettings.Landscape = True
            ' Druckeinstellungen für DIN A4
            Dim paperSize As New PaperSize("A4", 827, 1169) ' Größe in Hundertstel Zoll
            printDocument.DefaultPageSettings.PaperSize = paperSize
            ' Drucken starten
            pageCount = 1 ' Startseite
            printDocument.Print()
        End If
    End Sub

    Private Sub PrintPageHandler(sender As Object, e As PrintPageEventArgs)
        Dim utFont As New Font("Arial", 4.5)
        Dim headerFont As New Font("Arial", 12, FontStyle.Bold)
        Dim cellFont As New Font("Arial", 9)
        Dim columnHeaderFont As New Font("Segoe UI", 7.5) ' Neue Schriftart für die Spaltenüberschriften mit kleinerer Größe
        Dim rowHeight = cellFont.Height + 8 ' Erhöhen Sie die Zellenhöhe um 10 Pixel
        Dim cellPadding = 5
        Dim tableWidth = e.MarginBounds.Width
        Dim tableHeight = e.MarginBounds.Height - headerFont.Height
        Dim tableRect As New RectangleF(e.MarginBounds.Left - 60, e.MarginBounds.Top, tableWidth, tableHeight) ' Verschieben Sie die Tabelle um 2 cm nach links
        ' Untertitel zeichnen
        'Dim utx As Single = e.MarginBounds.Left - 40
        'Dim uty As Single = e.MarginBounds.Top - 10
        'e.Graphics.DrawString(untertitel, headerFont, Brushes.Maroon, utx, uty)

        ' Header zeichnen
        Dim headerX As Single = e.MarginBounds.Left
        Dim headerY As Single = e.MarginBounds.Top - 60 ' Verschieben Sie den Header um 2 cm nach oben
        e.Graphics.DrawString(title, headerFont, Brushes.Blue, headerX, headerY)

        ' Tabelle zeichnen
        tableRect.Y += headerFont.Height + 50 - 100 ' Erhöhen Sie den Abstand zwischen Header und Tabelle um 10 Pixel und verschieben Sie die Tabelle um 2 cm nach oben
        Dim columnX As Single = tableRect.Left
        Dim columnY As Single = tableRect.Y
        For i As Integer = 0 To dataTable.Columns.Count - 1
            Dim columnWidth = (tableWidth + 30) / dataTable.Columns.Count + 6 ' Erhöhen Sie die Spaltenbreite um 2 Buchstaben
            Dim columnName = dataTable.Columns(i).ColumnName
            e.Graphics.FillRectangle(Brushes.LightGray, columnX, columnY, CSng(columnWidth), rowHeight)
            e.Graphics.DrawRectangle(Pens.Black, columnX, columnY, CSng(columnWidth), rowHeight)
            e.Graphics.DrawString(columnName, columnHeaderFont, Brushes.Blue, columnX + cellPadding, columnY + cellPadding) ' Verwenden Sie die neue Schriftart für die Spaltenüberschrift
            columnX += CSng(columnWidth)
        Next
        ' Tabelleninhalt zeichnen
        columnY += rowHeight
        Dim startRow = (pageCount - 1) * RowsPerPage
        Dim endRow = Math.Min(startRow + RowsPerPage, dataTable.Rows.Count)
        Dim isAlternateRow As Boolean = False
        For i As Integer = startRow To endRow - 1
            columnX = tableRect.Left
            For j As Integer = 0 To dataTable.Columns.Count - 1
                Dim columnWidth = (tableWidth + 30) / dataTable.Columns.Count + 6 ' Erhöhen Sie die Spaltenbreite um 2 Buchstaben
                Dim cellValue = dataTable.Rows(i)(j).ToString()
                ' Farbe für die Zeile festlegen
                Dim rowBrush As Brush = If(isAlternateRow, Brushes.LightGray, Brushes.White)
                e.Graphics.FillRectangle(rowBrush, columnX, columnY, CSng(columnWidth), rowHeight)
                e.Graphics.DrawRectangle(Pens.Black, columnX, columnY, CSng(columnWidth), rowHeight)
                If Double.TryParse(cellValue, 0.0) AndAlso Double.Parse(cellValue) < 0 Then
                    ' Negative Werte in Rot drucken
                    e.Graphics.DrawString(cellValue, cellFont, Brushes.Red, columnX + cellPadding, columnY + cellPadding)
                Else
                    e.Graphics.DrawString(cellValue, cellFont, Brushes.Black, columnX + cellPadding, columnY + cellPadding)
                End If
                columnX += CSng(columnWidth)
            Next
            columnY += rowHeight
            isAlternateRow = Not isAlternateRow
        Next
        'Prüfen, ob weitere Zeilen vorhanden sind oder Seiten überschritten werden
        If endRow < dataTable.Rows.Count Then
            e.HasMorePages = True
            pageCount += 1
        Else
            e.HasMorePages = False
            ' Druckbericht abschließen
            pageCount = 1
        End If
    End Sub
End Class