﻿Public Class XmlManager
    ' Event, das bei einem XML-Datei Fehler ausgelöst wird
    Public Event XMLError As EventHandler(Of String)
    ' Event, das bei erfolgreichem Lesen der XML-Daten ausgelöst wird
    Public Event XMLRead As EventHandler(Of DataTable)
    ' Event, das bei erfolgreichem Speichern der XML-Daten ausgelöst wird
    Public Event XMLSave As EventHandler
    ' Event, das bei erfolgreichem anlegen der XML-Daten ausgelöst wird
    Public Event XMLCreate As EventHandler
    ' DataSet und DataTables für die Verwaltung der XML-Daten.
    ' DataSet zur Speicherung der XML-Daten.
    Public dsWeather As New DataSet
    ' DataTable zur Speicherung der Wetterdaten.
    Public dtWeatherData As New DataTable

    ''' <summary>
    ''' Lädt die XML-Daten aus der angegebenen Datei.
    ''' </summary>
    ''' <param name="monthfile">Der Dateipfad der XML-Datei.</param>
    Public Sub LoadXMLData(monthfile As String)
        If File.Exists(monthfile) Then
            Try
                Debug.WriteLine("XML-Dateiname/Pfad: " & monthfile)
                dsWeather.Clear()
                dsWeather.ReadXml(monthfile)
                If dsWeather.Tables.Count > 0 Then
                    dtWeatherData = dsWeather.Tables(0)
                    ' Zum Testen: 
                    Dim tableName As String = dsWeather.Tables(0).TableName
                    Debug.WriteLine("DataTable: " & tableName)
                    ' Test Ende
                Else
                    Debug.WriteLine("Die XML-Datei enthält keine Daten.")
                End If
                RaiseEvent XMLRead(Me, dtWeatherData)
            Catch ex As Exception
                RaiseEvent XMLError(Me, ex.Message)
                Debug.WriteLine("Fehler beim Laden der XML-Daten: " & ex.Message)
                Debug.WriteLine("Details: " & ex.StackTrace)
            End Try
        End If
    End Sub

    ''' <summary>
    ''' Speichert die XML-Daten in der angegebenen Datei.
    ''' </summary>
    ''' <param name="monthfile">Der Dateipfad, in dem die XML-Daten gespeichert werden sollen.</param>
    Public Sub SaveXMLData(monthfile As String)
        If File.Exists(monthfile) Then
            Try
                Debug.WriteLine("XML-Dateiname/Pfad: " & monthfile)
                ' Formatierung der Werte in den relevanten Spalten der DataTable
                For Each row As DataRow In dtWeatherData.Rows
                    For columnIndex As Integer = 3 To dtWeatherData.Columns.Count - 1
                        Dim value As Double = CDbl(row(columnIndex))
                        Dim formattedValue As String = value.ToString("0.0")
                        row(columnIndex) = formattedValue
                    Next
                Next
                dsWeather.WriteXml(monthfile)
                Debug.WriteLine("XML-Daten erfolgreich gespeichert.")
                RaiseEvent XMLSave(Me, EventArgs.Empty)
                ' Zum Testen: Den gekürzten Inhalt des ersten <WeatherData>-Tags ausgeben
                Dim tableName As String = dsWeather.Tables(0).TableName
                Debug.WriteLine("DataTable: " & tableName)
                ' Test Ende
            Catch ex As Exception
                RaiseEvent XMLError(Me, ex.Message)
                Debug.WriteLine("Fehler beim Speichern der XML-Datei: " & ex.Message)
            End Try
        End If
    End Sub

    ''' <summary>
    ''' Schutzprüfung beim versehendlichen schließen des Programms.
    ''' </summary>
    Public Function DatenGeändert(dgvData As DataGridView) As Boolean
        ' Überprüfen, ob es Änderungen im DataGridView gibt
        ' Durchlaufe die Zeilen im DataGridView
        For Each row As DataGridViewRow In dgvData.Rows
            ' Überprüfen, ob die Zeile als bearbeitet markiert ist
            If row.IsNewRow OrElse row.Cells.Cast(Of DataGridViewCell)().Any(Function(c) c.IsInEditMode) Then
                ' Es gibt eine neue Zeile oder eine Zelle in Bearbeitung, also wurden Daten geändert
                Return True
            End If
        Next
        ' Keine Änderungen gefunden
        Return False
    End Function

    ''' <summary>
    ''' Erstellt eine neue XML-Datei fals diese noch nicht vorhanden.
    ''' </summary>
    ''' <param name="monthfile">Der Dateipfad, in dem die XML-Daten gespeichert werden sollen.</param>
    Public Sub CreateNewMonth(monthfile As String, daysInMonth As Integer)
        If Not File.Exists(monthfile) Then
            Try
                Dim newMonthData As New DataSet()
                Dim weatherDataTable As New DataTable("WeatherData")
                ' Spalten für die Wetterdaten hinzufügen
                weatherDataTable.Columns.Add("Datum", GetType(String))
                weatherDataTable.Columns.Add("WetterTag", GetType(String))
                weatherDataTable.Columns.Add("WetterNacht", GetType(String))
                weatherDataTable.Columns.Add("RegenTag", GetType(Double))
                weatherDataTable.Columns.Add("SchneeTag", GetType(Double))
                weatherDataTable.Columns.Add("SchneeNacht", GetType(Double))
                weatherDataTable.Columns.Add("LuftfminTag", GetType(Double))
                weatherDataTable.Columns.Add("LuftfmaxTag", GetType(Double))
                weatherDataTable.Columns.Add("LuftfminNacht", GetType(Double))
                weatherDataTable.Columns.Add("LuftfmaxNacht", GetType(Double))
                weatherDataTable.Columns.Add("TempTag", GetType(Double))
                weatherDataTable.Columns.Add("TempNacht", GetType(Double))
                weatherDataTable.Columns.Add("WindTag", GetType(Double))
                weatherDataTable.Columns.Add("WindNacht", GetType(Double))
                ' Einträge für jeden Tag im Monat hinzufügen
                For day As Integer = 1 To daysInMonth
                    Dim newRow As DataRow = weatherDataTable.NewRow()
                    newRow("Datum") = day.ToString("00") & "-" & Path.GetFileNameWithoutExtension(monthfile)
                    newRow("WetterTag") = String.Empty
                    newRow("WetterNacht") = String.Empty
                    newRow("RegenTag") = 0
                    newRow("SchneeTag") = 0
                    newRow("SchneeNacht") = 0
                    newRow("LuftfminTag") = 0
                    newRow("LuftfmaxTag") = 0
                    newRow("LuftfminNacht") = 0
                    newRow("LuftfmaxNacht") = 0
                    newRow("TempTag") = 0
                    newRow("TempNacht") = 0
                    newRow("WindTag") = 0
                    newRow("WindNacht") = 0
                    weatherDataTable.Rows.Add(newRow)
                Next
                ' DataTable dem DataSet hinzufügen
                newMonthData.Tables.Add(weatherDataTable)
                ' DataSet in XML-Datei schreiben
                newMonthData.WriteXml(monthfile)
                Debug.WriteLine("Neue XML-Datei für Monat " & monthfile & " wurde erstellt.")
                ' Event auslösen
                RaiseEvent XMLCreate(Me, EventArgs.Empty)
            Catch ex As Exception
                RaiseEvent XMLError(Me, ex.Message)
                Debug.WriteLine("Fehler beim Erstellen der XML-Datei für Monat " & monthfile & ": " & ex.Message)
            End Try
        End If
    End Sub
End Class